package net.sf.dc.console.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;

import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;

public class FieldSelectorField extends JComponent implements ISettingsComponent {

    private LinkedList components = new LinkedList();
    private int module = -1; 
    
    public FieldSelectorField(int module) {
        this.module = module;
        
        buildComponent();        
    }
    
    
    public Object getValue() {
        return getEnabledFields();
    }
    
    public void setValue(Object o) {
        if (o instanceof int[]) {
            setSelectedFields((int[]) o);
        }
    }

    public int[] getEnabledFields() {
        Collection fields = new ArrayList();
        int counter = 0;
        for (Iterator iter = components.iterator(); iter.hasNext(); counter++) {
            JCheckBox checkBox = (JCheckBox) iter.next();
            if (checkBox.isSelected()) {
                fields.add(new Integer(counter));
            }
        }
        
        int[] indices = new int[fields.size()];
        counter = 0;
        for (Iterator iter = fields.iterator(); iter.hasNext(); counter++) {
            indices[counter] = ((Integer) iter.next()).intValue();
        }
        return indices;
    }
    
    public void setSelectedFields(int[] fields) {
        DCObject dco = DCModules.getModule(module).getDCObject();
        
        int i = 0;
        for (Iterator iter = components.iterator(); iter.hasNext(); i++) {
            DCFieldDefinition field = dco.getField(i);
            JCheckBox checkBox = (JCheckBox) iter.next();
            checkBox.setSelected(false);
            if (fields.length == 0) {
                checkBox.setSelected(!field.isHidden());
            } else {
                for (int j = 0; j < fields.length; j++) {
                    if (i == fields[j]) 
                        checkBox.setSelected(true);
                }
            }
        }
    }
    
    private void buildComponent() {
        setLayout(Layout.getGBL());
        Color color = DcSettings.getValueAsColor(Repository.settings.stColor);
        
        JPanel panel = new JPanel(false);
        panel.setLayout(Layout.getGBL());
        panel.setBackground(color);
        
        DCObject dco = DCModules.getModule(module).getDCObject();
        
        int x = 0;
        int y = 0;
        
        DCFieldDefinition[] fields = dco.getFields();
        for (int i = 0; i < fields.length; i++) {
            DCFieldDefinition field = fields[i];
            
            JCheckBox checkBox = ComponentFactory.getCheckBox(field.getLabel());
            checkBox.setBackground(color);
            components.addLast(checkBox);
            
            panel.add(checkBox, Layout.getGBC(x++, y, 1, 1, 1.0, 1.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets( 0, 10, 0, 0), 0, 0));            
            
            if (x == 4) {
                x = 0;
                y++;
            }
        }
        
        add(panel, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                   new Insets( 0, 0, 0, 0), 0, 0));            
        
    }
    
    public void setBackground(Color color) {
        super.setBackground(color);
        Component[] comps = getComponents();
        for (int i = 0; i < comps.length; i++) {
            JComponent component = (JComponent) comps[i];
            component.setBackground(color);
        }
        
        for (Iterator iter = components.iterator(); iter.hasNext(); ) {
            ((JComponent) iter.next()).setBackground(color);
        }
    }      
}
