/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.table.TableColumn;

import net.sf.dc.console.components.renderers.DCTableCellRenderer;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;
import net.sf.ext.util.Resources;

/**
 * Table in which the required fields for a module can be set.
 * Typically this component would be use in the Settings Panel
 * 
 * @see net.sf.ext.jflexsett.settings.ui.SettingsPanel
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class RequiredFieldsTable extends JComponent implements ISettingsComponent {
    
    private DCTable table;
    private final DCModule module;
    
    /**
     * Creates this panel for the provided module
     * @param module
     */
    public RequiredFieldsTable(DCModule module) {
        this.module = module;
        buildPanel();
    }


    
    /**
     * Sets the give field as required
     * @param value int array, required field indices
     */
    public void setValue(Object value) {
        table.setRowCount(0);
        
        String[] fields = new String[module.getDCObject().getDatabaseFieldNames().length -1];
        
        for (int i = 1; i < fields.length + 1; i++) {
            fields[i-1] = module.getDCObject().getDatabaseFieldName(i);
        }
        
        int[] requiredFieldIds = (int[]) value;
        
        for (int i = 0; i < fields.length; i++) {
            String[] row = new String[2];
            row[0] = fields[i];
            row[1] = Resources.getText(DCLabels.lblNo);
            for (int j = 0; j < requiredFieldIds.length; j++) {
                if (requiredFieldIds[j] == i + 1) {
                    row[1] = Resources.getText(DCLabels.lblYes);   
                }
            }   
            table.addRow(row);
        } 
    }    
    
    /**
     * Returns the required fields
     * @return required field indices array (int[]) as object 
     */
    public Object getValue() {
        Vector vFields = new Vector(0);
        for (int i = 0; i < table.getRowCount(); i++) {
            String sText = table.getValueAt(i, 2).toString();
            if (sText.equals(Resources.getText(DCLabels.lblYes))) {
                vFields.add(new Integer(i + 1));
            }
        }

        int[] fields = new int[vFields.size()];
        
        int i = 0;
        while (!vFields.isEmpty()) {
            fields[i++] = ((Integer) vFields.remove(0)).intValue();
        }
        
        DCFieldDefinition[] fieldDefs = module.getFieldDefinitions();
        for (int j = 0; j < fieldDefs.length; j++) {
            fieldDefs[j].setRequired(false);
        }
            
        for (int k = 0; k < fields.length; k++) {
            fieldDefs[fields[k]].setRequired(true);
        }        

        return fields;
    }
    
    private void buildPanel() {
        JPanel panelRequiredFields = new JPanel(false);
        panelRequiredFields.setLayout(Layout.getGBL());

        table = ComponentFactory.getDCTable(false, false);
        table.setPreferredScrollableViewportSize(new Dimension(402, 285));

        DCTableCellRenderer renderer = new DCTableCellRenderer();

        table.setColumnCount(2);
        TableColumn columnNew = table.getColumnModel().getColumn(0);
        columnNew.setHeaderValue(Resources.getText(DCLabels.lblFields));

        JTextField text = new JTextField();
        text.setEditable(false);
        columnNew.setCellEditor(new DefaultCellEditor(text));
        renderer.setBackground(ComponentFactory.getDisabledColor());
        columnNew.setCellRenderer(renderer);
        columnNew = table.getColumnModel().getColumn(1);
        columnNew.setHeaderValue(Resources.getText(DCLabels.lblRequiredYN));
        JComboBox comboBox = ComponentFactory.getRealYesNoComboBox();
        columnNew.setCellEditor(new DefaultCellEditor(comboBox));
        renderer  = new DCTableCellRenderer();
        columnNew.setCellRenderer(renderer);

        table.applyHeaders();
        
        JScrollPane scrollResults = new JScrollPane(table);
        scrollResults.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);        

        panelRequiredFields.add(scrollResults,  Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                                               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                                new Insets( 5, 5, 5, 5), 0, 0));    

        setLayout(Layout.getGBL());
        
        add(panelRequiredFields, Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                 new Insets( 0, 0, 0, 0), 0, 0));
    }    
}
