/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Image;
import java.awt.Insets;
import java.io.File;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.DCDialog;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.components.PictureField;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class AboutDialog extends DCDialog {
    
    /**
     * Initializes and creates this dialog, then it is shown
     */
    public AboutDialog(JFrame parent) {
        super(parent);
        
        buildDialog();
        
        setTitle("");
        Image img =IconLibrary._icoMain.getImage();
        img.flush();
        
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);
    }

    /**
     * Closes and disposes this dialog
     */
    public void close() {
        this.dispose();
    }

    /**
     * Created this dialog
     */
    private void buildDialog() {
        JTabbedPane tabbedPane = ComponentFactory.getTabbedPane();
        
        //**********************************************************
        //About panel
        //**********************************************************
        JPanel panelAbout = new JPanel(false);
        panelAbout.setLayout(Layout.getGBL());
        
        PictureField about = ComponentFactory.getPictureField(false, false, false);
        try {
            File file = new File("./icons_system/about.jpg");
            about.setValue(file.toURL());
            
            panelAbout.add(about, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
            new Insets(5, 5, 5, 5), 0, 0));            
        } catch (Exception exp) {
            Messages.add(exp);
        }

        //**********************************************************
        //Credits
        //**********************************************************        
        JPanel panelCredits = new JPanel(false);
        panelCredits.setLayout(Layout.getGBL());
        
        PictureField credits = ComponentFactory.getPictureField(false, false, false);
        try {
            File file = new File("./icons_system/credits.jpg");
            credits.setValue(file.toURL());
            
            panelCredits.add(credits, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
            new Insets(5, 5, 5, 5), 0, 0));            
        } catch (Exception exp) {
            Messages.add(exp);
        }
        
        //**********************************************************
        //Main panel
        //**********************************************************
        tabbedPane.addTab(Resources.getText(DCLabels.lblAbout), panelAbout);
        tabbedPane.addTab(Resources.getText(DCLabels.lblCredits), panelCredits);

        
        getContentPane().setLayout(Layout.getGBL());
        getContentPane().add(tabbedPane,   Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
                                           GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                           new Insets(5, 5, 5, 5), 0, 0));

        setTitle(Resources.getText(DCMessages.msgQuestion));
        pack();
        setSize(new Dimension(430,450));
        setResizable(false);
        
        
    }
}
