/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.expert;

import java.awt.Cursor;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.swing.JLabel;
import javax.swing.JPanel;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.Repository;
import net.sf.dc.messages.Messages;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.wf.WorkFlow;

/**
 * Shows System Information like the amount of records in the database
 * per module, the driver used to connect to the database and the
 * database name
 * 
 * @author Jesper
 * @since 1.4
 * @version 1.6
 */
public class SystemInfoPanel extends JPanel {
    
    private JLabel recordsLabel = null;
    private JLabel softwareLabel = null;
    private JLabel softwareCountLabel = null;
    private JLabel movieLabel = null;
    private JLabel movieCountLabel = null;
    private JLabel musicFileLabel = null;
    private JLabel musicFileCountLabel = null;
    private JLabel audioCDLabel = null;
    private JLabel audioCDCountLabel = null;
    private JLabel totalLabel = null;
    private JLabel totalCountLabel = null;
    private JLabel databaseNameLabel = null;
    private JLabel databaseNameTextLabel = null;
    private JLabel databaseDriverLabel = null;
    private JLabel databaseDriverTextLabel = null;

    /**
     * Initializes and creates this panel
     */
    public SystemInfoPanel() {
        buildDialog();
        getData(); 
    }
    
    public void getData() {
    	int softwareCount = 0;
    	int movieCount = 0;
    	int musicFileCount = 0;
    	int audioCDCount = 0;
    	int totalCount = 0;
        String sSoftwareQuery = "SELECT count(*) FROM software";
        String sMovieQuery = "SELECT count(*) FROM movies";
        String sMusicFileQuery = "SELECT count(*) FROM musicfiles";
        String sAudioCDQuery = "SELECT count(*) FROM audiocds";
        ResultSet result;
        
        setCursor(new Cursor(Cursor.WAIT_CURSOR));

        try {
	        result = runQuery(sSoftwareQuery);
	        softwareCount = getRecordCount(result);
	        softwareCountLabel.setText(Integer.toString(softwareCount));
	        
	        result = runQuery(sMovieQuery);
	        movieCount = getRecordCount(result);
	        movieCountLabel.setText(Integer.toString(movieCount));
	        
	        result = runQuery(sMusicFileQuery);
	        musicFileCount = getRecordCount(result);
	        musicFileCountLabel.setText(Integer.toString(musicFileCount));
	        
	        result = runQuery(sAudioCDQuery);
	        audioCDCount = getRecordCount(result);
	        audioCDCountLabel.setText(Integer.toString(audioCDCount));
	        
	        totalCount = softwareCount + movieCount + musicFileCount + audioCDCount;
	        totalCountLabel.setText(Integer.toString(totalCount));
	    } catch(Exception ex) {
            Messages.add(ex.getMessage(), Messages._ERROR);
        	new MessageBox(ex.getMessage(), MessageBox._ERROR);
        }

        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    private ResultSet runQuery(String sQuery) {
        ResultSet result = null;
        
        try {
            WorkFlow wf = new WorkFlow();
            WorkFlow.initDatabase();

            result = wf.runQueryDirectUnclosed(sQuery);
        } catch (Exception ex) {
            Messages.add(ex.getMessage(), Messages._ERROR);
            new MessageBox(ex.getMessage(), MessageBox._ERROR);
        } 
        
        return result;
    }
    
    private int getRecordCount(ResultSet result) {
        int recordCount = 0;
        boolean empty = false;

        if (result != null) {
            try {
                result.isLast();
            } catch (Exception exp) {
                empty = true;
            }
        }

        try {
            if (!empty) {
                result.next();
                recordCount = result.getInt(result.getMetaData().getColumnName(1));
            }
        } catch (Exception ex) {
            recordCount = 0;
        }

        // close the result set
        try {
            result.close();
        } catch (SQLException sqlExp) {
            Messages.add(sqlExp);
        }

        return recordCount;
    }
    
    /**
     * Created this dialog
     */
    private void buildDialog() {
        //**********************************************************
        //Database panel
        //**********************************************************
        JPanel panelDatabase = new JPanel(false);
        panelDatabase.setLayout(Layout.getGBL());
        
        recordsLabel = new JLabel("Records in database:");
        recordsLabel.setFont(ComponentFactory.getSystemFont());
        softwareLabel = new JLabel("Software");
        softwareCountLabel = new JLabel("");
        softwareCountLabel.setAlignmentX(JLabel.RIGHT_ALIGNMENT);
        movieLabel = new JLabel("Movie");
        movieCountLabel = new JLabel("");
        movieCountLabel.setAlignmentX(JLabel.RIGHT_ALIGNMENT);
        musicFileLabel = new JLabel("MP3");
        musicFileCountLabel = new JLabel("");
        musicFileCountLabel.setAlignmentX(JLabel.RIGHT_ALIGNMENT);
        audioCDLabel = new JLabel("AudioCD");
        audioCDCountLabel = new JLabel("");
        audioCDCountLabel.setAlignmentX(JLabel.RIGHT_ALIGNMENT);
        totalLabel = new JLabel("Total");
        totalCountLabel = new JLabel("");
        totalCountLabel.setAlignmentX(JLabel.RIGHT_ALIGNMENT);
        databaseNameLabel = new JLabel("Database");
        databaseNameTextLabel = new JLabel(DcSettings.getValueAsString(Repository.settings.stConnectionString));
        databaseDriverLabel = new JLabel("Database driver");
        databaseDriverTextLabel = new JLabel(DcSettings.getValueAsString(Repository.settings.stDatabaseDriver));
        
        panelDatabase.add(recordsLabel, Layout.getGBC( 0, 0, 1, 1, 2.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets( 0, 5, 0, 0), 0, 0));
        
        panelDatabase.add(softwareLabel, Layout.getGBC( 0, 1, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        panelDatabase.add(softwareCountLabel, Layout.getGBC( 1, 1, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        panelDatabase.add(movieLabel, Layout.getGBC( 0, 2, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        panelDatabase.add(movieCountLabel, Layout.getGBC( 1, 2, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        panelDatabase.add(musicFileLabel, Layout.getGBC( 0, 3, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        panelDatabase.add(musicFileCountLabel, Layout.getGBC( 1, 3, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        panelDatabase.add(audioCDLabel, Layout.getGBC( 0, 4, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        panelDatabase.add(audioCDCountLabel, Layout.getGBC( 1, 4, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        panelDatabase.add(totalLabel, Layout.getGBC( 0, 5, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        panelDatabase.add(totalCountLabel, Layout.getGBC( 1, 5, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        panelDatabase.add(databaseNameLabel, Layout.getGBC( 0, 6, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        panelDatabase.add(databaseNameTextLabel, Layout.getGBC( 1, 6, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        panelDatabase.add(databaseDriverLabel, Layout.getGBC( 0, 7, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        panelDatabase.add(databaseDriverTextLabel, Layout.getGBC( 1, 7, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        //**********************************************************
        //Main panel
        //**********************************************************        
        add(panelDatabase, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
    }
}
