/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.io.File;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JMenu;

import net.sf.dc.console.DCMenuBar;
import net.sf.dc.console.book.BookCategoryForm;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.book.BookCategory;
import net.sf.dc.core.objects.reporting.ReportFieldDefinitions;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * The Book category module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 1.00
 */
public class BookCategoryModule extends DCModule {
    
    private final DCMenuBar menuBar = null;
    private BookCategoryForm form;
    
    /**
     * Initializes this module
     */
    public BookCategoryModule() {
        super(DCModules._BOOKCAT,
              Resources.getText(DCLabels.lblModuleBookCategory),
              "bookcategories",
              "soca",
              "bookcategories",
              false,
              false,
              false);
    }

    public DCMenuBar getMenuBar() {
        return menuBar;
    }    
    
    public ImageIcon getIcon() {
        return null;
    }
    
    public String getObjectName() {
        return Resources.getText(DCLabels.lblBookCategory);
    }
    
    /**
     * Create the insert and search panel
     */    
    protected void initializeUI() {
        form = new BookCategoryForm(DataCrow.mainFrame);
        insertPanel = form.insertCat;
        searchPanel = form.searchCat;
    }
    


    public void startImport() {}
    
    /**
     * Returns the form which holds the UI items for this Module.
     * In this, the Book Category module is an odd one. None
     * of the other modules have a specialized form.
     * 
     * The form consists of the insert and search panels
     */
    public BookCategoryForm getForm() {
        return form;    
    }

    public JMenu getToolsMenu() {
        return null;
    }  

    /**
     * Return the object managed by this module 
     * @see BookCategory
     */ 
    public DCObject getDCObject() {
        return new BookCategory();
    }
    
    public String getHiddenFieldsSettingsKey() {
        return Repository.settings.stHiddenFieldsBookCat;
    }

    public String getRequiredFieldsSettingsKey() {
        return Repository.settings.stBookCatRequiredFields;
    }
    
    /**
     * Returns null (no CD import process present for Book
     * Categories) 
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     * 
     * @return null
     */ 
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
                                              String nameOfCD) {
        return null;
    }  
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        return new ReportFieldDefinitions();
    }    
    
    /**
     * Initalizes all the fields
     */    
    protected void initializeFields() {
        fields = new DCFieldDefinition[3];
        int counter = 0;
        fields[BookCategory._ID] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblID),
                                        true,
                                        false,
                                        false,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");        
        fields[BookCategory._A_NAME] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblName),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Name");
        fields[BookCategory._B_ICON] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblIcon),
                                        true,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "Icon");        
    }   
    
    public DCObject[] getDefaultData() throws Exception {
        String link = "file://" + Utilities.getCurrentDirectory() + File.separator + 
                      "icons_user" + File.separator;
            
        String[][] values = {{"Action", link + "BangBoy.gif"},
                             {"Thriller", link + "Clawful.gif"},
                             {"Technical", link + "ZerToolz.gif"},
                             {"Fantasy", link + "LittleDraggin.gif"},
                             {"Romance", link + "Prophet.gif"},
                             {"Science Fiction", link + "FatBot.gif"},
                             {"Encyclopedia", link + "GottaDaTrash.gif"},
                             {"Comic Book", link + "ColonParen.gif"},
                             {"Study Book", link + "DontGotIt.gif"},
                             {"Movie", link + "Cactusususus.gif"},
                             {"Humor", link + "LeSmileyDeClassique.gif"}};

        DCObject[] objects = new DCObject[values.length];
        for (int i = 0; i < values.length; i++) {
            DCObject dco = new BookCategory();

            URL url = new URL(values[i][1]);
            String s = Utilities.fileToBase64String(url);

            dco.setValue(BookCategory._A_NAME, values[i][0]);
            dco.setValue(BookCategory._B_ICON, s);                            
            
            objects[i] = dco;
        }
        return objects;
    }
}
