/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.audiocd;

import javax.swing.ImageIcon;

import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.settings.DcSettings;

/**
 * The Audio Track Object
 * @see net.sf.dc.core.objects.DCObject
 *
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 1.50
 */
public class AudioTrack extends DCObject {

    public static final int _A_TRACKNUMBER = 1;
    public static final int _B_TITLE = 2;
    public static final int _C_LYRICS = 3;
    public static final int _D_PLAYLENGTH = 4;
    public static final int _E_ALBUM = 5;

    public AudioTrack() {
        super();
    }
    
    /**
     * Sets the module index / sets the ownership of this
     * data crow object  
     */    
    protected void setModuleIndex() {
        module = DCModules._AUDIOTRACK;
    }
    
    /**
     * Returns the icon for this object
     */    
    public ImageIcon getIcon() {
        return null;    
    }
    
    /**
     * Initialize all user settings on this data crow object 
     */    
    protected void initializeSettings() {
        DCFieldDefinition[] fields = getFields();
        
        int[] requiredFields = DcSettings.getValueAsIntegerArray(
                                  Repository.settings.stAudioTrackRequiredFields);
                                  
        for (int i = 0; i < requiredFields.length; i++) {
            fields[requiredFields[i]].setRequired(true);
        }
                                  
        int[] hiddenFields = DcSettings.getValueAsIntegerArray(
                                Repository.settings.stHiddenFieldsAudioTrack);   

        for (int i = 0; i < hiddenFields.length; i++) {
            fields[hiddenFields[i]].setHidden(true);
        }
    }
    
    public void checkIntegrity(boolean update) throws ValidationException {}
    
    /**
     * Create a UI representation for this object
     */
    public String toString() {
        return getValue(AudioTrack._ID).toString();
    }     
}
