/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http;

import java.net.URL;

import net.sf.dc.console.InternetSearchForm;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.book.Book;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.messages.Messages;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.StringUtils;

/**
 * Searches for software descriptions and screenshots on the Amazon site
 *
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class AmazonBookUpdate extends AmazonUpdate {

    public AmazonBookUpdate(InternetSearchForm ui,
                                String searchString, 
                                boolean broadband) {
        super(ui, searchString, broadband);
    }
    
    public void run() {
    	search();
    }
    
    /**
     * Searches on the Amazon.com website on the catagories software and
     * videogames. Returns a Vector of Strings. Each String describes 
     * a found item on the Amazon website
     */
    public void search() {
        try {
            String server = DcSettings.getValueAsString(Repository.settings.stAmazonServer);
            searchString = searchString.replaceAll(" ", "%20");
            URL url = new URL(server + "/onca/xml3?t=webservices-20&" +
                              "dev-t=1N37KF9JXJXVF9JEFB02&KeywordSearch=" + searchString +
                              "&mode=books&type=heavy&page=1&f=xml");
            super.search(url);
        } catch (Exception exp) {
            new MessageBox(exp.getMessage(), MessageBox._ERROR);
            ui.setSearchMode();
        }
    }

    /**
     * Parses information from a raw html or xml string 
     * @param xmlString the raw input
     */
    protected DCObject convertXmlToObject(String xmlString) {
        Book book = new Book();
        
        int indexBegin = 0;
        int indexEnd = 0;

        String productName = "";
        indexBegin = xmlString.indexOf("<ProductName>");
        if (indexBegin != -1) {
            indexBegin = indexBegin + 13;
            indexEnd = xmlString.indexOf("</ProductName>");
            productName = StringUtils.convertHttpString(xmlString.substring(indexBegin, indexEnd));
        }
        
        String amazonUrl = "";
        indexBegin = xmlString.indexOf("url=");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 5;
            indexEnd = xmlString.indexOf(">") - 1;
            amazonUrl = xmlString.substring(indexBegin, indexEnd);
        }        

        String sID = "";
        if (amazonUrl.indexOf("ASIN/") > 0) {
        	sID = amazonUrl.substring(amazonUrl.indexOf("ASIN/") + 5, amazonUrl.length());
        	sID = sID.substring(0, sID.indexOf("/"));
        }
        
        indexBegin = xmlString.indexOf("<ImageUrlLarge>");
        if (indexBegin != -1) {
            try {
            	String quality = broadband ? "LZZZZZZZ.jpg" : "MZZZZZZZ.jpg";
                
                URL url = new URL("http://images.amazon.com/images/P/" + sID + ".01." + quality);
                HttpConnection connection = new HttpConnection(url);
                book.setValue(Book._I_PICTUREFRONT, connection.getImageAsBase64());
                connection.close();
            } catch (Exception exp) {
                Messages.add(exp);
            }
        }        
        
        book.setValue(Book._A_TITLE, StringUtils.convertHttpString(productName));
        book.setValue(Book._B_DESCRIPTION, getDescription(xmlString));
        book.setValue(Book._C_PUBLISHER, getManufacturer(xmlString));
        book.setValue(Book._D_AUTHOR, getAuthors(xmlString));
        book.setValue(Book._E_RELEASEYEAR, getReleaseDate(xmlString));
        book.setValue(Book._F_WEBPAGE, amazonUrl); 
        book.setValue(Book._K_RATING, getRating(xmlString));

        return book;
    }
    
    private String getAuthors(String rawResult) {
        int indexBegin = 0;
        int indexEnd = 0;
        
        String platforms = "";
        indexBegin = rawResult.indexOf("<Authors>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 11;
            indexEnd = rawResult.indexOf("</Authors>");
            platforms = rawResult.substring(indexBegin, indexEnd);

            String temp = "";
            indexBegin = platforms.indexOf("<Author>");
            while(indexBegin > -1) {
                indexBegin = indexBegin + 10;
                indexEnd = platforms.indexOf("</Author>");

                temp = temp + platforms.substring(indexBegin, indexEnd) + " ";

                indexEnd = indexEnd + 11;
                platforms = platforms.substring(indexEnd, platforms.length());
                indexBegin = platforms.indexOf("<Authors>");
            }
            platforms = temp;
        }
        return StringUtils.convertHttpString(platforms);
    }


}
