package net.sf.dc.settings;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import net.sf.dc.console.components.ColorSelector;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.FontSelector;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.reporting.ReportFieldDefinitions;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCTooltips;
import net.sf.ext.jflexsett.settings.Setting;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.jflexsett.settings.SettingsFile;
import net.sf.ext.jflexsett.settings.SettingsGroup;
import net.sf.ext.jflexsett.settings.ui.SettingsView;
import net.sf.ext.util.Resources;


public class ReportSettings {
    
    public static final int _CARD_REPORT = 0;
    public static final int _LIST_REPORT = 1;
    
    public static final String _ReportText =  Resources.getText(DCLabels.lblReportTextSetting);
    public static final String _ReportLabel = Resources.getText(DCLabels.lblReportLabelSetting);    
    public static final String _ReportTextBorderColor =  Resources.getText(DCLabels.lblReportTextBorderColor);
    public static final String _ReportLabelBorderColor =  Resources.getText(DCLabels.lblReportLabelBorderColor);
    public static final String _ReportHeader = Resources.getText(DCLabels.lblReportHeaderSetting);
    public static final String _ReportHeaderFont = Resources.getText(DCLabels.lblReportHeaderFont);
    public static final String _ReportHeaderColor = Resources.getText(DCLabels.lblReportHeaderColor);
    public static final String _ReportHeaderBackgroundColor = Resources.getText(DCLabels.lblReportHeaderBackgroundColor);
    public static final String _ReportFooter = Resources.getText(DCLabels.lblReportFooterSetting);
    public static final String _ReportFooterFont = Resources.getText(DCLabels.lblReportFooterFont);
    public static final String _ReportFooterColor = Resources.getText(DCLabels.lblReportFooterColor);
    public static final String _ReportFooterBackgroundColor = Resources.getText(DCLabels.lblReportFooterBackgroundColor);
    public static final String _ReportTextFontSetting = Resources.getText(DCLabels.lblReportTextFontSetting);
    public static final String _ReportTextColorSetting = Resources.getText(DCLabels.lblReportTextColorSetting);
    public static final String _ReportTextBackgroundColorSetting = Resources.getText(DCLabels.lblReportTextBackgroundColorSetting);
    public static final String _ReportLabelFontSetting = Resources.getText(DCLabels.lblReportLabelFontSetting);
    public static final String _ReportLabelColorSetting = Resources.getText(DCLabels.lblReportLabelColorSetting);
    public static final String _ReportLabelBackgroundColorSetting = Resources.getText(DCLabels.lblReportLabelBackgroundColorSetting);    
    
    private static Map cardSettingsMap = new HashMap();
    private static Map listSettingsMap = new HashMap();
    
    private static int currentModule = -1;
    private static int currentReportType = -1;
    
    public ReportSettings() {
    }

    protected void initializeSettings(DCModule module) {
        Settings listSettings = new Settings();
        createGroups(listSettings);
        createSettings(listSettings, _LIST_REPORT);
        File listSettingsFile = new File("./data/" + module.getTableName() + "_list_report.properties");
        listSettings.setSettingsFile(listSettingsFile);
        SettingsFile.load(listSettings);
        listSettingsMap.put(module.getName(), listSettings);               
        
        Settings cardSettings = new Settings();
        createGroups(cardSettings);
        createSettings(cardSettings, _CARD_REPORT);
        File cardSettingsFile = new File("./data/" + module.getTableName() + "_card_report.properties");
        cardSettings.setSettingsFile(cardSettingsFile);
        SettingsFile.load(cardSettings);
        cardSettingsMap.put(module.getName(), cardSettings);
    }
    
    public static void setCurrentReport(int index, int module) {
        currentReportType = index;
        currentModule = module;
    }
    
    private static Settings getCurrentSettings() {
        DCModule module = DCModules.getModule(currentModule);
        String moduleName = DCModules.getModule(currentModule).getName();
        Settings settings;
        
        if (!cardSettingsMap.keySet().contains(moduleName)) {
            new ReportSettings().initializeSettings(module);  
        }
        
        if (currentReportType == _CARD_REPORT) {
            settings = (Settings) cardSettingsMap.get(moduleName);
        } else {
            settings = (Settings) listSettingsMap.get(moduleName);
        }
        return settings;
    }
    
    public static void save() {
        for (Iterator iter = listSettingsMap.values().iterator(); iter.hasNext(); ) {
            Settings settings = (Settings) iter.next();
            SettingsFile.save(settings);
        }

        for (Iterator iter = cardSettingsMap.values().iterator(); iter.hasNext(); ) {
            Settings settings = (Settings) iter.next();
            SettingsFile.save(settings);
        }
    }
    
    public static SettingsView getView() {
        Settings settings = getCurrentSettings();
        SettingsView view = new SettingsView(null, "Report Settings Panel", settings);
        view.setDisclaimer("icons_system" + File.separator + "logo.jpg");
        view.setSize(new Dimension(875, 450));
        Image image = IconLibrary._icoSettings.getImage();
        view.setIconImage(image);
        image.flush(); 
        return view;
    }
    
    public static ReportFieldDefinitions getValueAsReportFieldDefinitions(String key) {
        return getCurrentSettings().getValueAsReportFieldDefinitions(key);
    }    
    
    public static String[] getValueAsStringArray(String key) {
        return getCurrentSettings().getValueAsStringArray(key);
    }
    
    public static int[] getValueAsIntegerArray(String key) {
        return getCurrentSettings().getValueAsIntegerArray(key);
    }

    public static Font getValueAsFont(String key) {
        return getCurrentSettings().getValueAsFont(key);
    }    
    
    public static int getValueAsInt(String key) {
        return getCurrentSettings().getValueAsInt(key);
    }

    public static boolean getValueAsBoolean(String key) {
        return getCurrentSettings().getValueAsBoolean(key);
    }


    public static Color getValueAsColor(String key) {
        return getCurrentSettings().getValueAsColor(key);
    }
    
    public static String getValueAsString(String key) {
        return getCurrentSettings().getValueAsString(key);
    }

    public static void setValue(String key, Object value) {
        getCurrentSettings().setValue(key, value);
    }

    public static void setStringAsValue(String key, String value) {
        getCurrentSettings().setStringAsValue(key, value);
    }     

    /**
    * Creates the settings group (with their hierarchy)
    */
    protected void createGroups(Settings settings) {
        SettingsGroup reportTextGroup = new SettingsGroup(_ReportText, "dc.Reporting.TextSetting");
        SettingsGroup reportLabelGroup = new SettingsGroup(_ReportLabel, "dc.Reporting.LabelSetting");
        SettingsGroup reportHeaderGroup = new SettingsGroup(_ReportHeader, "dc.Reporting.HeaderSetting");
        SettingsGroup reportFooterGroup = new SettingsGroup(_ReportFooter, "dc.Reporting.FooterSetting");
        
        reportTextGroup.addChild(new SettingsGroup(_ReportTextFontSetting, "dc.Reporting.TextSetting"));
        reportTextGroup.addChild(new SettingsGroup(_ReportTextColorSetting, "dc.Reporting.TextSetting"));
        reportTextGroup.addChild(new SettingsGroup(_ReportTextBackgroundColorSetting, "dc.Reporting.TextSetting"));
        reportTextGroup.addChild(new SettingsGroup(_ReportTextBorderColor, "dc.Reporting.TextSetting"));
        
        reportLabelGroup.addChild(new SettingsGroup(_ReportLabelFontSetting, "dc.Reporting.LabelSetting"));
        reportLabelGroup.addChild(new SettingsGroup(_ReportLabelColorSetting, "dc.Reporting.LabelSetting"));
        reportLabelGroup.addChild(new SettingsGroup(_ReportLabelBackgroundColorSetting, "dc.Reporting.LabelSetting"));
        reportLabelGroup.addChild(new SettingsGroup(_ReportLabelBorderColor, "dc.Reporting.LabelSetting"));

        reportFooterGroup.addChild(new SettingsGroup(_ReportFooterFont, "dc.Reporting.FooterSetting"));
        reportFooterGroup.addChild(new SettingsGroup(_ReportFooterColor, "dc.Reporting.FooterSetting"));
        reportFooterGroup.addChild(new SettingsGroup(_ReportFooterBackgroundColor, "dc.Reporting.FooterSetting"));

        reportHeaderGroup.addChild(new SettingsGroup(_ReportHeaderFont, "dc.Reporting.HeaderSetting"));
        reportHeaderGroup.addChild(new SettingsGroup(_ReportHeaderColor, "dc.Reporting.HeaderSetting"));
        reportHeaderGroup.addChild(new SettingsGroup(_ReportHeaderBackgroundColor, "dc.Reporting.HeaderSetting"));
        
        settings.addGroup(_ReportText, reportTextGroup);
        settings.addGroup(_ReportLabel, reportLabelGroup);
        settings.addGroup(_ReportHeader, reportHeaderGroup);
        settings.addGroup(_ReportFooter, reportFooterGroup);
    }
    
    private void createSettings(Settings settings, int reportType) {
        if (reportType == _CARD_REPORT) {
            settings.addSetting(_ReportText,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.reportsettings.stCreateIndex,
                                Boolean.TRUE,
                                ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblCreateIndex)),
                                Resources.getText(DCTooltips.tpCreateIndex),
                                "",
                                false,
                                true));   
            settings.addSetting(_ReportText,
                    new Setting(Repository.datatypes._INTEGER,
                                Repository.reportsettings.stLabelPosition,
                                new Integer(1),
                                ComponentFactory.getReportTextPositionCombo(),
                                Resources.getText(DCTooltips.tpTextPosition),
                                Resources.getText(DCLabels.lblTextPosition),
                                true,
                                true));     
        }

        settings.addSetting(_ReportTextFontSetting,                           // Group key
                new Setting(Repository.datatypes._FONT,                       // Data type
                            Repository.reportsettings.stTextFont,             // Key
                            new Font("Arial", Font.PLAIN, 11),                   // Default value
                            new FontSelector(),                               // UI Component
                            Resources.getText(DCTooltips.tpReportTextFont),   // Help text
                            "",                                               // Label text
                            false,                                             // show label ?
                            true));                                           // Add to panel ?    
        settings.addSetting(_ReportTextBackgroundColorSetting,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stTextBackgroundcolor,
                            new Color(247,247,221),
                            new ColorSelector(false, Repository.reportsettings.stTextBackgroundcolor),
                            Resources.getText(DCTooltips.tpReportTextBackground),
                            "",
                            true,
                            true));
        settings.addSetting(_ReportTextColorSetting,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stTextColor,
                            Color.BLACK,
                            new ColorSelector(false, Repository.reportsettings.stTextColor),
                            Resources.getText(DCTooltips.tpReportTextColor),
                            "",
                            true,
                            true));
        settings.addSetting(_ReportTextBorderColor,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stTextBorderColor,
                            Color.BLACK,
                            new ColorSelector(false, Repository.reportsettings.stTextBorderColor),
                            Resources.getText(DCTooltips.tpReportTextBorderColor),
                            "",
                            true,
                            true));        
        settings.addSetting(_ReportLabelFontSetting,
                new Setting(Repository.datatypes._FONT,
                            Repository.reportsettings.stLabelFont,
                            new Font("Arial", Font.BOLD, 11),
                            new FontSelector(),
                            Resources.getText(DCTooltips.tpReportLabelFont),
                            "",
                            false,
                            true));    
        settings.addSetting(_ReportLabelBackgroundColorSetting,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stLabelBackgroundColor,
                            new Color(240,240,191),
                            new ColorSelector(false, Repository.reportsettings.stLabelBackgroundColor),
                            Resources.getText(DCTooltips.tpReportLabelBackground),
                            "",
                            true,
                            true));
        settings.addSetting(_ReportLabelColorSetting,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stLabelColor,
                            Color.BLACK,
                            new ColorSelector(false, Repository.reportsettings.stLabelColor),
                            Resources.getText(DCTooltips.tpReportLabelColor),
                            "",
                            true,
                            true));
        settings.addSetting(_ReportLabelBorderColor,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stLabelBorderColor,
                            Color.BLACK,
                            new ColorSelector(false, Repository.reportsettings.stLabelBorderColor),
                            Resources.getText(DCTooltips.tpReportLabelBorderColor),
                            "",
                            true,
                            true));         
        settings.addSetting(_ReportHeader,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stShowHeader,
                            Boolean.TRUE,
                            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblShowHeader)),
                            Resources.getText(DCTooltips.tpShowHeader),
                            "",
                            false,
                            true));
        settings.addSetting(_ReportHeader,
                new Setting(Repository.datatypes._STRING,
                            Repository.reportsettings.stHeaderValue,
                            "Report Generated by Data Crow",
                            ComponentFactory.getShortTextField(200),
                            Resources.getText(DCTooltips.tpHeaderValue),
                            Resources.getText(DCLabels.lblHeaderValue),
                            true,
                            true));              
        settings.addSetting(_ReportFooter,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stShowFooter,
                            Boolean.TRUE,
                            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblShowFooter)),
                            Resources.getText(DCTooltips.tpShowFooter),
                            "",
                            false,
                            true));    
        settings.addSetting(_ReportFooter,
                new Setting(Repository.datatypes._STRING,
                            Repository.reportsettings.stFooterValue,
                            "page",
                            ComponentFactory.getShortTextField(200),
                            Resources.getText(DCTooltips.tpFooterValue),
                            Resources.getText(DCLabels.lblFooterValue),
                            true,
                            true));
        settings.addSetting(_ReportFooterFont,
                new Setting(Repository.datatypes._FONT,
                            Repository.reportsettings.stFooterFont,
                            new Font("dialog.plain", Font.BOLD, 11),
                            new FontSelector(),
                            Resources.getText(DCTooltips.tpReportFooterFont),
                            "",
                            false,
                            true));         
        settings.addSetting(_ReportFooterColor,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stFooterColor,
                            Color.BLACK,
                            new ColorSelector(false, Repository.reportsettings.stFooterColor),
                            Resources.getText(DCTooltips.tpReportFooterColor),
                            "",
                            true,
                            true));         
        settings.addSetting(_ReportFooterBackgroundColor,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stFooterBackgroundColor,
                            Color.WHITE,
                            new ColorSelector(false, Repository.reportsettings.stFooterBackgroundColor),
                            Resources.getText(DCTooltips.tpReportFooterBackgroundColor),
                            "",
                            true,
                            true));     
        settings.addSetting(_ReportHeaderFont,
                new Setting(Repository.datatypes._FONT,
                            Repository.reportsettings.stHeaderFont,
                            new Font("dialog.plain", Font.BOLD, 11),
                            new FontSelector(),
                            Resources.getText(DCTooltips.tpReportHeaderFont),
                            "",
                            false,
                            true));         
        settings.addSetting(_ReportHeaderColor, 
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stHeaderColor,
                            Color.BLACK,
                            new ColorSelector(false, Repository.reportsettings.stHeaderColor),
                            Resources.getText(DCTooltips.tpReportHeaderColor),
                            "",
                            true,
                            true));         
        settings.addSetting(_ReportHeaderBackgroundColor,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stHeaderBackgroundColor,
                            Color.WHITE,
                            new ColorSelector(false, Repository.reportsettings.stHeaderBackgroundColor),
                            Resources.getText(DCTooltips.tpReportHeaderBackgroundColor),
                            "",
                            true,
                            true));      
        
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stTextBorderBottom,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblTextBorderBottom)),
                            "",
                            "",
                            false,
                            true));            
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stTextBorderTop,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblTextBorderTop)),
                            "",
                            "",
                            false,
                            true));            
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stTextBorderLeft,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblTextBorderLeft)),
                            "",
                            "",
                            false,
                            true));            
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stTextBorderRight,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblTextBorderRight)),
                            "",
                            "",
                            false,
                            true));   
        settings.addSetting(_ReportLabel,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stLabelBorderBottom,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblLabelBorderBottom)),
                            "",
                            "",
                            false,
                            true));            
        settings.addSetting(_ReportLabel,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stLabelBorderTop,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblLabelBorderTop)),
                            "",
                            "",
                            false,
                            true));            
        settings.addSetting(_ReportLabel,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stLabelBorderLeft,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblLabelBorderLeft)),
                            "",
                            "",
                            false,
                            true));            
        settings.addSetting(_ReportLabel,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stLabelBorderRight,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblLabelBorderRight)),
                            "",
                            "",
                            false,
                            true));                    
        
        
        // INVISIBLE SETTINGS
        if (currentModule != -1) {
            DCModule module = DCModules.getModule(currentModule);
            ReportFieldDefinitions definitions = module.getReportFieldDefinitions();
            
            settings.addSetting(_ReportText,
                    new Setting(Repository.datatypes._REPORTFIELDDEFINITIONS,
                                Repository.reportsettings.stFieldDefinitions,
                                definitions,
                                null,
                                "",
                                "",
                                false,
                                false));             
        }
        
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._STRING,
                            Repository.reportsettings.stTarget,
                            "",
                            null,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.reportsettings.stReportType,
                            new Integer(0),
                            null,
                            "",
                            "",
                            false,
                            false));           
    }    
}