package net.sf.ext.mediamanager;

/**
 * @(#)FileProperties.java 1.0 06.06.05 (dd.mm.yy)
 * 
 * Copyright (2003) Mediterranean
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Boston, MA 02111.
 * 
 * Contact: mediterranean@users.sourceforge.net
 */

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.StringTokenizer;

abstract class FileProperties {

    private String _subtitles = "";
    private String _videoResolution = "";
    private String _videoCodec = "";
    private int _videoRate = 0;
    private int _videoBitRate = 0;
    private int _duration = -1;
    private String _audioCodec = "";
    private int _audioRate = 0;
    private int _audioBitRate = 0;
    private int _audioChannels = 0;
    private String _location = "";
    private String _container = "";
    private String _mediaType = "";
    private ArrayList metaData;

    /**
     * Returns the subtitles.
     */
    protected String getSubtitles() {
        return _subtitles;
    }

    /**
     * Returns the resolution.
     */
    protected String getVideoResolution() {
        return _videoResolution;
    }

    /**
     * Returns the video codec.
     */
    protected String getVideoCodec() {
        return _videoCodec;
    }

    /**
     * Returns the video rate.
     */
    protected int getVideoRate() {
        return _videoRate;
    }

    /**
     * Returns the video bit rate.
     */
    protected int getVideoBitRate() {
        return _videoBitRate;
    }

    /**
     * Returns the duration.
     */
    protected int getDuration() {
        return _duration;
    }

    /**
     * Returns the audio codec.
     */
    protected String getAudioCodec() {
        return _audioCodec;
    }

    /**
     * Returns the audio rate.
     */
    protected int getAudioRate() {
        return _audioRate;
    }

    /**
     * Returns the audio bit rate.
     */
    protected int getAudioBitRate() {
        return _audioBitRate;
    }

    /**
     * Returns the audio channels.
     */
    protected int getAudioChannels() {
        return _audioChannels;
    }

    /**
     * Sets the subtitles.
     */
    protected void setSubtitles(String subtitles) {
        _subtitles = subtitles;
    }

    /**
     * Sets the resolution.
     */
    protected void setVideoResolution(String videoResolution) {
        _videoResolution = videoResolution;
    }

    /**
     * Sets the video codec (vids handler).
     */
    protected void setVideoCodec(String videoCodec) {
        _videoCodec = videoCodec;
    }

    /**
     * Sets the video rate.
     */
    protected void setVideoRate(int videoRate) {
        _videoRate = videoRate;
    }

    /**
     * Sets the video bit rate.
     */
    protected void setVideoBitRate(int videoBitRate) {
        _videoBitRate = videoBitRate;
    }

    /**
     * Sets the duration.
     */
    protected void setDuration(int duration) {
        _duration = duration;
    }

    /**
     * Sets the audio codec (auds handler).
     */
    protected void setAudioCodec(String audioCodec) {
        _audioCodec = audioCodec;
    }

    /**
     * Sets the audio rate.
     */
    protected void setAudioRate(int audioRate) {
        _audioRate = audioRate;
    }

    /**
     * Sets the audio bit rate.
     */
    protected void setAudioBitRate(int audioBitRate) {
        _audioBitRate = audioBitRate;
    }

    /**
     * Sets the audio channels.
     */
    protected void setAudioChannels(int audioChannels) {
        _audioChannels = audioChannels;
    }

    /**
     * Sets the container.
     */
    protected void setContainer(String container) {
        _container = container;
    }

    /**
     * Returns the container.
     */
    protected String getContainer() {
        return _container;
    }

    /**
     * Sets the Media Type.
     */
    protected void setMediaType(String mediaType) {
        _mediaType = mediaType;
    }

    /**
     * Returns the Media Type.
     */
    protected String getMediaType() {
        return _mediaType;
    }

    /**
     * Sets the meta data ArrayList.
     */
    protected void setMetaData(ArrayList metaData) {
        this.metaData = metaData;
    }

    /**
     * Returns the meta data ArrayList.
     */
    protected ArrayList getMetaData() {
        return metaData;
    }

    /**
     * Processes a file from the given DataInputStream.
     */
    protected void process(RandomAccessFile dataStream) throws Exception {
    }

    /**
     * Reads an unsigned 8-bit integer.
     */
    protected int readUnsignedByte(byte[] b, int offset) throws Exception {
        return b[offset];
    }

    /**
     * Reads an unsigned 16-bit integer.
     */
    protected int readUnsignedInt16(byte[] b, int offset) throws Exception {
        return (b[offset] | (b[offset + 1] << 8));
    }

    /**
     * Reads an unsigned 32-bit integer.
     */
    protected int readUnsignedInt32(byte[] b, int offset) throws Exception {
        return (readUnsignedInt16(b, offset) | (readUnsignedInt16(b, offset + 2) << 16));
    }

    /**
     * Returns a 16-bit integer.
     */
    protected int getUnsignedInt16(int byte1, int byte2) throws Exception {
        return (byte2 | (byte1 << 8));
    }

    /**
     * Returns a 16-bit integer.
     */
    protected int getUnsignedInt16(byte byte1, byte byte2) throws Exception {
        return (new Byte(byte2).intValue() | new Byte(byte1).intValue() << 8);
    }

    /**
     * Returns an unsigned 32-bit integer.
     */
    protected int getUnsignedInt32(byte byte1, byte byte2) throws Exception {
        return (new Byte(byte2).intValue() | new Byte(byte1).intValue() << 16);
    }

    /**
     * Returns an unsigned 32-bit integer.
     */
    protected int getUnsignedInt32(int byte1, int byte2) throws Exception {
        return (byte1 | byte2 << 16);
    }

    /**
     * Reads an unsigned byte and returns its int representation.
     */
    protected int readUnsignedByte(RandomAccessFile dataStream) throws Exception {
        int data = dataStream.readUnsignedByte();
        if (data == -1) {
            throw new Exception("Unexpected end of stream.");
        }
        return data;
    }

    /**
     * Reads n unsigned bytes and returns it in an int[n].
     */
    protected int[] readUnsignedBytes(RandomAccessFile dataStream, int n)
            throws Exception {
        int[] data = new int[n];
        for (int i = 0; i < data.length; i++) {
            data[i] = readUnsignedByte(dataStream);
        }
        return data;
    }

    /**
     * Reads an unsigned 16-bit integer.
     */
    protected int readUnsignedInt16(RandomAccessFile dataStream) throws Exception {
        return (readUnsignedByte(dataStream) | (readUnsignedByte(dataStream) << 8));
    }

    /**
     * Reads an unsigned 32-bit integer.
     */
    protected int readUnsignedInt32(RandomAccessFile dataStream) throws Exception {
        return (readUnsignedInt16(dataStream) | (readUnsignedInt16(dataStream) << 16));
    }

    /**
     * Discards n bytes.
     */
    protected void skipBytes(RandomAccessFile dataStream, int n) throws Exception {
        readUnsignedBytes(dataStream, n);
    }

    /**
     * Reverses the byte order
     */
    int changeEndianness(int num) {
        return (num >>> 24) | (num << 24)
                | ((num << 8) & 0x00FF0000 | ((num >> 8) & 0x0000FF00));
    }

    /**
     * Returns the ascii value of id
     */
    String fromByteToAscii(int id, int numberOfBytes) throws Exception {
        StringBuffer buffer = new StringBuffer(4);

        for (int i = 0; i < numberOfBytes; i++) {
            int c = id & 0xff;
            buffer.append((char) c);
            id >>= 8;
        }
        return new String(buffer);
    }

    /**
     * Returns the decimal value of a specified bumber of bytes from a specific
     * part of a byte.
     */
    public int getDecimalValue(int[] bits, int start, int stop, boolean printBits) {
        String dec = "";
        for (int i = start; i >= stop; i--) {
            dec += bits[i];
        }

        return Integer.parseInt(dec, 2);
    }

    /**
     * Returns an array containing the bits from the value.
     */
    int[] getBits(int value, int numberOfBytes) {

        int[] bits = new int[numberOfBytes * 8];

        for (int i = bits.length - 1; i >= 0; i--) {
            bits[i] = (value >>> i) & 1;
        }
        return bits;
    }

    /**
     * Searchs in the inputStream stream the name following the string id
     * (seperated by a \t).
     */
    protected String findName(InputStream stream, String id) throws Exception {
        BufferedReader reader = new BufferedReader(new InputStreamReader(stream));
        String line = null;

        while ((line = reader.readLine()) != null) {
            if (line.length() > 0) {
                StringTokenizer tokenizer = new StringTokenizer(line, "\t");
                if (tokenizer.countTokens() > 0
                        && id.compareToIgnoreCase(tokenizer.nextToken()) == 0) {
                    return tokenizer.nextToken();
                }
            }
        }
        return "";
    }
}
